<?php
/**
 * @file
 * Integration with Mercado Pago SDK.
 */

/**
 * Define the configuration form of the payment Mercado Pago.
 */
function mercado_pago_payment_sdk_settings_form($settings = NULL) {
  $form = array();

  $form['client_id'] = array(
    '#type' => 'textfield',
    '#title' => t('Identificación del Client (CLIENT ID)'),
    '#description' => t('Número de cuenta. Averígualo en la configuración avanzada de MercadoPago.'),
    '#default_value' => (isset($settings['client_id'])) ? trim($settings['client_id']) : '',
    '#required' => TRUE,
  );

  $form['client_secret'] = array(
    '#type' => 'textfield',
    '#title' => t('Clave Secreta del Client (CLIENT SECRET)'),
    '#description' => t('Averígualo en la configuración avanzada de MercadoPago.'),
    '#default_value' => (isset($settings['client_secret'])) ? trim($settings['client_secret']) : '',
    '#required' => TRUE,
  );

  $form['information'] = array(
    '#type' => 'textarea',
    '#title' => t('Información para el usuario'),
    '#description' => t('Información que se desplegará al momento de seleccionar Mercado Pago en las formas de pago durante el proceso de Checkout.'),
    '#default_value' => (isset($settings['information'])) ? trim($settings['information']) : '',
    '#required' => TRUE,
  );

  return $form;
}


/**
 * Payment method callback: adds a message and CSS to the submission form.
 */
function mercado_pago_payment_sdk_submit_form($payment_method, $pane_values, $checkout_pane, $order) {
  $form['mp_information'] = array(
    '#markup' => $payment_method['settings']['information'],
    '#attached' => array(
      'css' => array(drupal_get_path('module', 'mercado_pago_payment') . '/theme/mercado_pago_payment.css'),
    ),
  );

  return $form;
}


/**
 * Implementation of CALLBACK_redirect_form().
 */
function mercado_pago_payment_sdk_redirect_form($form, &$form_state, $order, $payment_method) {
  $url_return = url('checkout/' . $order->order_id . '/payment/return/' .
    $order->data['payment_redirect_key'], array('absolute' => TRUE));
  $url_back = url('checkout/' . $order->order_id . '/payment/back/' .
    $order->data['payment_redirect_key'], array('absolute' => TRUE));

  drupal_add_js(
    "function mercado_pago_onreturn(json) {
      if (json.collection_status=='approved' ||
          json.collection_status=='pending' ||
          json.collection_status=='in_process'){    
        document.location.href='$url_return';
      } else {
        document.location.href='$url_back';
      } 
    }
    function fireEvent(obj,evt){
        var fireOnThis = obj;
        if( document.createEvent ) {
           var evObj = document.createEvent('MouseEvents');
           evObj.initEvent( evt, true, false );
           fireOnThis.dispatchEvent( evObj );
        } else if( document.createEventObject ) {
          var evObj = document.createEventObject();
          fireOnThis.fireEvent( 'on' + evt, evObj );
        }
     }
    fireEvent(document.getElementById(\"btnPagar\"), 'click')
  ", array('type' => 'inline', 'scope' => 'footer'));

  // Currency code for Mercado pago standar.
  $wrapper = entity_metadata_wrapper('commerce_order', $order);
  $currency_code = $wrapper->commerce_order_total->currency_code->value();
  $mp_curreny_code = $currency_code;
  $codes = mercado_pago_payment_currency_codes();

  if (isset($codes[$currency_code])) {
    $mp_curreny_code = $codes[$currency_code];
  }

  // Getting unity price.
  $amount = $wrapper->commerce_order_total->amount->value();

  // Getting cart name and cart surname.
  $name_billing = $wrapper->commerce_customer_billing->value();

  if (isset($name_billing->commerce_customer_address[LANGUAGE_NONE][0]['first_name']) &&
    isset($name_billing->commerce_customer_address[LANGUAGE_NONE][0]['last_name'])) {

    $cart_name = $name_billing->commerce_customer_address[LANGUAGE_NONE][0]['first_name'];
    $cart_surname = $name_billing->commerce_customer_address[LANGUAGE_NONE][0]['last_name'];

  }
  else {
    list($cart_name, $cart_surname) = explode(' ',
      $name_billing->commerce_customer_address[LANGUAGE_NONE][0]['name_line'] . ' ', 2);
  }

  // Setting preference for MP.
  $preference = array(
    "items" => array(
      array(
        "title" => t('Order @order_number at @store',
          array(
            '@order_number' => $order->order_number,
            '@store' => variable_get('site_name', url('<front>', array('absolute' => TRUE))),
          )
        ),
        "quantity" => 1,
        "currency_id" => $mp_curreny_code,
        "unit_price" => round(commerce_currency_amount_to_decimal($amount, $currency_code), 2),
        "id" => $order->order_number,
      ),
    ),
    "payer" => array(
      "name" => $cart_name,
      "surname" => $cart_surname,
      "email" => $order->mail,
    ),
    "back_urls" => array(
      "success" => $url_return,
      "failure" => $url_back,
      "pending" => $url_return,
    ),
    "external_reference" => $order->order_id . '-' . REQUEST_TIME,
  );

  // Ensure a default value for the payment_method setting.
  $settings = $payment_method['settings'];
  $info = libraries_load('mp_sdk');

  $mp = new MP($settings["client_id"], $settings["client_secret"]);
  $preference_result = $mp->create_preference($preference);

  // Usaremos la versión con link de MP, que parece ser la más versatil.
  // TODO: Implementar esto para que se pueda elegir "mp-mode" que se quiera.
  $form["mp_link_button"] = array(
    '#markup' => implode('',
      array(
        '<a href="',
        $preference_result['response']['init_point'],
        '" id="btnPagar" name="MP-payButton" mp-mode="modal"',
        ' class="lightblue-M-Ov-ArOn" onreturn="mercado_pago_onreturn">Pagar</a>',
        '<script type="text/javascript" src="https://www.mercadopago.com/org-img/jsapi/mptools/buttons/render.js"></script>',
      )
    ),
  );

  return $form;
}

/**
 * Processes an incoming IPN.
 */
function _mercado_pago_payment_sdk_validate_ipn($payment_method = NULL) {
  drupal_add_http_header('Content-Type', 'text/plain');
  $ipn = $_GET;

  if (empty($ipn)) {
    watchdog('marcado_pago_sdk', 'IPN URL accessed with no POST data submitted.', array(), WATCHDOG_WARNING);
    drupal_add_http_header('Status', '401 ERROR');
    echo "Error:174";
    exit;
  }

  if (!isset($ipn["topic"]) || $ipn["topic"] != "payment") {
    watchdog('marcado_pago_sdk', 'IPN URL accessed with no "topic" or not set to "payment" on POST data submitted.',
      array(), WATCHDOG_WARNING);
    drupal_add_http_header('Status', '401 ERROR');
    echo "Error:182";
    exit;
  }

  if (!isset($ipn["id"])) {
    watchdog('marcado_pago_payment_sdk', 'IPN URL accessed with no id on POST data submitted.',
      array(), WATCHDOG_WARNING);
    drupal_add_http_header('Status', '401 ERROR');
    echo "Error:190";
    exit;
  }

  // If no payment_method, load default instance.
  if (empty($payment_method)) {
    $payment_method = commerce_payment_method_instance_load(
      "mercado_pago_payment_sdk|commerce_payment_mercado_pago_payment_sdk");
  }

  $settings = $payment_method['settings'];

  $info = libraries_load('mp_sdk');
  $mp = new MP($settings["client_id"], $settings["client_secret"]);

  try {
    $payment_info = $mp->get_payment_info('' . $ipn["id"]);
  }
  catch (Exception $e) {
    watchdog('marcado_pago_sdk', 'An error ocurred when processing payment with id = %id.<br /> Message: <pre>!message</pre>',
      array('%id' => $ipn["id"], '!message' => $e->getMessage()), WATCHDOG_WARNING);
    drupal_add_http_header('Status', '401 ERROR');
    echo 'Caught exception: ',  $e->getMessage(), "\n";
    exit;
  }

  if ($payment_info['status'] != '200') {
    watchdog('marcado_pago_sdk', 'An error ocurred when processing payment with id = %id.',
      array('%id' => $ipn["id"]), WATCHDOG_WARNING);
    drupal_add_http_header('Status', '401 ERROR');
    echo "Error:199";
    exit;
  }

  list($order_id) = explode('-', $payment_info['response']['collection']['external_reference']);

  if (!$order_id || empty($order_id)) {
    watchdog('mercado_pago_sdk', 'La información enviada no pudo ser procesada, el parametro "external_reference" es no valido. <pre>%posted_info</pre>',
      array('%posted_info' => check_plain(print_r($payment_info, TRUE))), WATCHDOG_INFO);
    echo "Error:208";
    exit;
  }

  $order = commerce_order_load($order_id);
  $wrapper = entity_metadata_wrapper('commerce_order', $order);
  $currency_code = $wrapper->commerce_order_total->currency_code->value();

  $transaction = NULL;
  $transactions = commerce_payment_transaction_load_multiple(array(), array('remote_id' => $ipn["id"]));
  if (!empty($transactions)) {
    $transaction = current($transactions);
  }

  if ($transaction == NULL) {
    $transaction = commerce_payment_transaction_new('mercado_pago_payment_sdk', $order->order_id);
    $transaction->instance_id = $payment_method['instance_id'];
  }

  $transaction->remote_id = $ipn["id"];
  $transaction->amount = commerce_currency_decimal_to_amount($payment_info['response']['collection']['transaction_amount'], $currency_code);
  $transaction->currency_code = $currency_code;
  $transaction->payload[REQUEST_TIME] = $payment_info;

  switch ($payment_info['response']['collection']['status']) {
    case 'approved':
      $transaction->status = COMMERCE_PAYMENT_STATUS_SUCCESS;
      $transaction->message = 'Pago acreditado';
      break;

    case 'cancelled':
      $transaction->status = COMMERCE_PAYMENT_STATUS_FAILURE;
      $transaction->message = 'Mercadopago cancelado';
      break;

    case 'in_mediation':
      $transaction->status = COMMERCE_PAYMENT_STATUS_PENDING;
      $transaction->message = 'El pago entró en disputa';
      break;

    case 'in_process':
      $transaction->status = COMMERCE_PAYMENT_STATUS_PENDING;
      $transaction->message = 'Su Mercadopago está pendiente de acreditación';
      break;

    case 'pending':
      $transaction->status = COMMERCE_PAYMENT_STATUS_PENDING;
      $transaction->message = 'Mercadopago pendiente, aún no se realiza deposito';
      break;

    case 'refunded':
      $transaction->status = COMMERCE_PAYMENT_STATUS_FAILURE;
      $transaction->message = 'Pago reintegrado a la contraparte';
      break;

    case 'rejected':
      $transaction->status = COMMERCE_PAYMENT_STATUS_FAILURE;
      $transaction->message = 'Mercadopago rechazado';
      break;

    case 'uninitialized':
      $transaction->status = COMMERCE_PAYMENT_STATUS_FAILURE;
      $transaction->message = 'Se abandonó el pago en el medio del proceso';
      break;

    default:
      // No action.
      break;

  }

  commerce_payment_transaction_save($transaction);
  // $posted_info['transaction_id'] = $transaction->transaction_id;
  watchdog('mercado_pago_sdk', 'El IPN de MercadoLibre fue procesado para la orden @order_number con transacción ID @txn_id.',
    array(
      '@txn_id' => $transaction->transaction_id,
      '@order_number' => $order->order_number,
    ),
    WATCHDOG_INFO);
  drupal_add_http_header('Status', '200 OK');
  echo "Success";
}
